@@ -287,6 +287,7 @@ void    finish_rule (int mach, int variable_trail_rule, int headcnt, int trailcn
 		add_action ("YY_RULE_SETUP\n");
 
 	line_directive_out(NULL, 1);
+        add_action("[[");
 }
 
 
@@ -498,6 +499,8 @@ int     mkor (int first, int second)
 		}
 	}
 
+	firstst[first] = MIN(firstst[first], firstst[second]);
+
 	finalst[first] = orend;
 	return first;
 }
